"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionAsync = actionAsync;
function _config() {
  const data = require("@expo/config");
  _config = function () {
    return data;
  };
  return data;
}
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function () {
    return data;
  };
  return data;
}
function _xdl() {
  const data = require("xdl");
  _xdl = function () {
    return data;
  };
  return data;
}
function _log() {
  const data = _interopRequireDefault(require("../../../log"));
  _log = function () {
    return data;
  };
  return data;
}
function _getRemoteVersionsForSdk() {
  const data = require("../../../utils/getRemoteVersionsForSdk");
  _getRemoteVersionsForSdk = function () {
    return data;
  };
  return data;
}
function _profileMethod() {
  const data = require("../../utils/profileMethod");
  _profileMethod = function () {
    return data;
  };
  return data;
}
function _validateDependenciesVersions() {
  const data = require("../../utils/validateDependenciesVersions");
  _validateDependenciesVersions = function () {
    return data;
  };
  return data;
}
function _explain() {
  const data = require("./depedencies/explain");
  _explain = function () {
    return data;
  };
  return data;
}
function _windows() {
  const data = require("./windows");
  _windows = function () {
    return data;
  };
  return data;
}
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
async function validateSupportPackagesAsync(sdkVersion) {
  const versionsForSdk = await (0, _getRemoteVersionsForSdk().getRemoteVersionsForSdk)(sdkVersion);
  const supportPackagesToValidate = ['expo-modules-autolinking', '@expo/config-plugins', '@expo/prebuild-config'];
  let allPackagesValid = true;
  for (const pkg of supportPackagesToValidate) {
    const version = versionsForSdk[pkg];
    if (version) {
      const isVersionValid = await (0, _explain().warnAboutDeepDependenciesAsync)({
        name: pkg,
        version
      });
      if (!isVersionValid) {
        allPackagesValid = false;
      }
    }
  }
  return allPackagesValid;
}

// Ensures that a set of packages
async function validateIllegalPackagesAsync() {
  const illegalPackages = ['@unimodules/core', '@unimodules/react-native-adapter', 'react-native-unimodules'];
  let allPackagesLegal = true;
  for (const pkg of illegalPackages) {
    const isPackageAbsent = await (0, _explain().warnAboutDeepDependenciesAsync)({
      name: pkg
    });
    if (!isPackageAbsent) {
      allPackagesLegal = false;
    }
  }
  return allPackagesLegal;
}
async function actionAsync(projectRoot, options) {
  await (0, _windows().warnUponCmdExe)();
  const {
    exp,
    pkg
  } = (0, _profileMethod().profileMethod)(_config().getConfig)(projectRoot);
  if (_xdl().Versions.gteSdkVersion(exp, '46.0.0')) {
    _log().default.error(`'expo-cli doctor' is not supported for SDK 46 and higher. Use 'npx expo-doctor' instead.`);
    process.exitCode = 1;
    return;
  }
  let foundSomeIssues = false;

  // Only use the new validation on SDK +45.
  if (_xdl().Versions.gteSdkVersion(exp, '45.0.0')) {
    if (!(await validateSupportPackagesAsync(exp.sdkVersion))) {
      foundSomeIssues = true;
    }
  }
  if (_xdl().Versions.gteSdkVersion(exp, '44.0.0')) {
    if (!(await validateIllegalPackagesAsync())) {
      foundSomeIssues = true;
    }
  }
  if (!(await (0, _profileMethod().profileMethod)(_validateDependenciesVersions().validateDependenciesVersionsAsync)(projectRoot, exp, pkg, options.fixDependencies))) {
    foundSomeIssues = true;
  }

  // note: this currently only warns when something isn't right, it doesn't fail
  await _xdl().Doctor.validateExpoServersAsync(projectRoot);
  if ((await _xdl().Doctor.validateWithNetworkAsync(projectRoot)) !== _xdl().Doctor.NO_ISSUES) {
    foundSomeIssues = true;
  }
  if (foundSomeIssues) {
    process.exitCode = 1;
  } else {
    _log().default.log(_chalk().default.green(`🎉 Didn't find any issues with the project!`));
  }
}
//# sourceMappingURL=doctorAsync.js.map